import camera
import network
import time
import gc
import jasnosc

from machine import Pin, PWM

latarka = PWM(Pin(4), 100)
latarka.duty(0)

SETPOINT = 40

#taśma
Kp = 1.2
Ki = 6
Kd = 0.001

#pudelko war. poczatkowe nr 1
#Kp = 0.2
#Ki = 1
#Kd = 0.002

#pudelko war. poczatkowe nr 2
#Kp = 0.05
#Ki = 0.5
#Kd = 0.001

previous_error = 0
integral = 0
dt = 0.32

# Funkcja regulatora PID
def pid_controller(setpoint, measured_value):
    global previous_error, integral
    
    # Obliczanie błędu
    error = setpoint - measured_value
    
    # Składowa proporcjonalna
    P = Kp * error
    
    # Składowa całkowa
    integral += error * dt
    I = Ki * integral
    
    # Składowa różniczkowa
    derivative = (error - previous_error) / dt
    D = Kd * derivative
    
    # Sygnał sterujący
    control_signal = P + I + D
    
    # Aktualizacja błędu poprzedniego
    previous_error = error
    
    return control_signal


try:
    camera.init(0, format=camera.GRAYSCALE, framesize=camera.FRAME_QQVGA, xclk_freq=camera.XCLK_10MHz, fb_location=camera.PSRAM)
    print('camera initialazed')
    
    while True:
        buffer = camera.capture()
        
        average_brightness = jasnosc.calculate_average_brightness(buffer)
        print(average_brightness)
        
        buffer = None  # Usuń dane bufora
        gc.collect()   # Wymuś zbieranie pamięci
        
        control_signal = pid_controller(SETPOINT, average_brightness)

        #print("control_signal:", control_signal)
        control_signal = round(control_signal)
        if control_signal < 0:
            control_signal = 0
        elif control_signal > 1023:
            control_signal = 1023
        #print("control_signal:", control_signal)
        
        latarka.duty(control_signal)
        time.sleep(0.2)
            
except Exception as e:
    print('An issue occurred')
    
finally:
    camera.deinit()
    print('Camera deinitialized')
